/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;

/**
 * A Panel that uses dubble-buffering to avoid flashing.
 *
 * @author Shmulik London
 */
public class DoubleBufferedPanel extends Panel {

    private Graphics bufferGraphics;
    private Dimension bufferDimension;
    private transient Image bufferImage;
    private transient Image backgroundImage;
    private boolean backgroundChanged = true;

    /**
     * Updates the display of the component.
     */
    public void update(Graphics g) {
        if ( (bufferGraphics == null)
             || !(bufferDimension.equals(getSize()))) {
            bufferDimension = getSize();
            bufferImage = createImage(getSize().width, getSize().height);
            bufferGraphics = bufferImage.getGraphics();
            backgroundImage = createImage(getSize().width, getSize().height);
            paintBackground(backgroundImage.getGraphics());
        }
        if (backgroundChanged) {
            Graphics backgroundGraphics = backgroundImage.getGraphics();
            backgroundGraphics.clearRect(0,0,getSize().width,getSize().height);
            paintBackground(backgroundGraphics);
        }
        bufferGraphics.drawImage(backgroundImage,0,0,this);
        paint(bufferGraphics);
        g.drawImage(bufferImage, 0, 0, this);
    }

    /**
     * Updates the background of the component.
     * This method should be called each time the
     * background painting has changed.
     */
    protected void updateBackground() {
        backgroundChanged=true;
        repaint();
    }

    /**
     * Paints a constant background for the component.
     * Use <TT>updateBackground()</TT> when the background
     * has changed.
     */
    public void paintBackground(Graphics g) {
    }
}
