/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;

/**
 * @author Shmulik London
 */
public class GroundLayout implements LayoutManager {

    /**
     * The left alignment variable.
     */
    public static final int LEFT 	= 0;

    /**
     * The right alignment variable.
     */
    public static final int CENTER 	= 1;

    /**
     * The right alignment variable.
     */
    public static final int RIGHT 	= 2;

    int align;
    int hgap;

    /**
     * Constructs a new GroundLayout.
     */
    public GroundLayout() {
	    this(CENTER, 20);
    }

    /**
     * Constructs a new GroundLayout.
     */
    public GroundLayout(int align) {
	    this(align, 20);
    }

    /**
     * Constructs a new GroundLayout.
     */
    public GroundLayout(int align, int hgap) {
    	this.align = align;
    	this.hgap = hgap;
    }

    /**
     * Returns the alignment value for this layout, one of LEFT,
     * CENTER, or RIGHT.
     */
    public int getAlignment() {
	    return align;
    }

    /**
     * Sets the alignment value for this layout.
     * @param align the alignment value, one of LEFT, CENTER, or RIGHT.
     */
    public void setAlignment(int align) {
	    this.align = align;
    }

    /**
     * Returns the horizontal gap between components.
     */
    public int getHgap() {
	    return hgap;
    }

    /**
     * Sets the horizontal gap between components.
     * @param hgap the horizontal gap between components
     */
    public void setHgap(int hgap) {
	    this.hgap = hgap;
    }

    /**
     * Adds the specified component to the layout. Not used by this class.
     * @param name the name of the component
     * @param comp the the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {
    }

    /**
     * Removes the specified component from the layout. Not used by
     * this class.
     * @param comp the component to remove
     */
    public void removeLayoutComponent(Component comp) {
    }

    /**
     * Returns the preferred dimensions for this layout given the components
     * in the specified target container.
     * @param target the component which needs to be laid out
     * @see Container
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container target) {
    	Dimension dim = new Dimension(0, 0);
    	return dim;
    }

    /**
     * Returns the minimum dimensions needed to layout the components
     * contained in the specified target container.
     * @param target the component which needs to be laid out
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container target) {
    	Dimension dim = new Dimension(0, 0);
    	return dim;
    }

    /**
     * Lays out the container. This method will actually reshape the
     * components in the target in order to satisfy the constraints of
     * the BorderLayout object.
     * @param target the specified component being laid out.
     * @see Container
     */
    public void layoutContainer(Container target) {
    	Insets insets = target.getInsets();
    	int nmembers = target.getComponentCount();

        int maxHeight = 0;
        int totalWidth = hgap;

    	for (int i = 0 ; i < nmembers ; i++) {
    	    Component m = target.getComponent(i);
    	    if (m.isVisible()) {
        		Dimension d = m.getPreferredSize();
        		m.setSize(d.width, d.height);
    	        maxHeight = Math.max(maxHeight,m.getSize().height);
    	        totalWidth += m.getSize().width + hgap;
    	    }
    	}

    	int x = insets.left + hgap +
    	    (target.getSize().width-insets.left-insets.right-totalWidth)/2;
    	int y = insets.top+(target.getSize().height-insets.top+insets.bottom-maxHeight)/2;

    	for (int i=0; i<nmembers; i++) {
    	    Component m = target.getComponent(i);
    	    if (m.isVisible()) {

                m.setLocation(x,y+maxHeight-m.getSize().height);
                x+=m.getSize().width+hgap;
            }
        }
    }

}
