/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package logo;

import java.awt.*;

/**
 * An image of a logo turtle. This is a component that is
 * used visualize the turtle.
 * @author Shmulik London
 */
public class TurtleImage extends Component {

    // The number of existing turtles
    private static int numberOfTurtles;

    // The direction of the turtle
    private double teta;

    // The ridial width of the turtle triangle
    private double gamma = Math.PI/6;

    // The plane on which the turtle lives
    private LogoPlane plane;

    // The color in which the turtle is drawing
    private Color penColor;

    /**
     * Constructs a new TurtleImage
     */
    public TurtleImage() {
        numberOfTurtles++;
        setSize(20,20);
        plane = LogoWindow.getDefaultWindow().getPlane();
        plane.addTurtle(this);
        penColor = Color.black;
    }

    /**
     * Paints the turtle.
     */
    public void paint(Graphics g) {
        int xm = getSize().width/2;
        int ym = getSize().height/2;
        double r = Math.min(getSize().height,getSize().width)/2;

        int[] xs = {
            (int)(xm+r*Math.cos(teta)),
            (int)(xm+r*Math.cos(teta+Math.PI-gamma)),
            (int)(xm+r*Math.cos(teta+Math.PI+gamma))
        };

        int[] ys = {
            (int)(ym-r*Math.sin(teta)),
            (int)(ym-r*Math.sin(teta+Math.PI-gamma)),
            (int)(ym-r*Math.sin(teta+Math.PI+gamma))
        };

        g.setColor(Color.red);
        g.fillPolygon(xs,ys,3);
        g.setColor(Color.blue);
        g.drawPolygon(xs,ys,3);
    }

    /**
     * Sets the angle of the turtle.
     */
    public void setAngle(double radians) {
        teta = radians;
        repaint();
    }

    /**
     * Moves the turtle by a given offset.
     * @param dx,dy The offset
     * @param pen If true leave a line behind the turtle.
     */
    public void moveBy(int dx, int dy, boolean pen) {
        Point location = getLocation();
        moveTo(location.x+dx, location.y+dy, pen);
    }
    
    /**
     * Moves the turtle to a new location.
     * @param x,y The new location.
     * @param pen If true leave a line behind the turtle.
     */
    public void moveTo(int x, int y, boolean pen) {
        Point from = getCenter();
        setLocation(x, y);
        if (pen) {
            plane.addLineSegment(from,getCenter(),penColor);
        }
        repaint();
    }

    /**
     * Returns the center point of the turtle image.
     */
    public Point getCenter() {
        return new Point(getLocation().x+getSize().width/2,
                         getLocation().y+getSize().height/2);
    }

}