/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

public class Frog {

    // The position of the frog
    private int position;

    // The name of the frog
    private String name;

    /**
     * Creates a new Frog.
     */
    public Frog() {
        this(null);
    }

    /**
     * Creates a new Frog.
     * @param name The name given to the frog.
     */
    public Frog(String name) {
        this.name = name;
        FrogRiddleVisualization.visualizeFrogCreation(this);
    }

    /**
     * Jump by a given number of steps.
     * @param steps The number of steps.
     */
    public void jumpBy(int steps) {
        FrogRiddleVisualization.visualizeFrogJump(this,steps);
        position+=steps;
    }

    /**
     * Jump to a given position.
     * @param position The new position.
     */
    public void jumpTo(int position) {
        int step = position - this.position;
        jumpBy(step);
    }

    /**
     * Sets the position of the frog. No jump is made the frog
     * just appears in the new position.
     * @param position The new position.
     */
    public void setPosition(int position) {
        this.position = position;
        FrogRiddleVisualization.visualizeFrogPositionChange(this);
    }

    /**
     * Returns the position of the frog.
     */
    public final int getPosition() {
        return position;
    }

    /**
     * Make the frog rest for a given number of seconds.
     */
    public void rest(int seconds) {
        try {
            Thread.sleep(seconds*1000);
        } catch (InterruptedException ie) {
        }
    }

}