/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.awt.image.*;
import jclip.*;
import clips.motion.*;

public class FrogImage extends ImageSetComponent {

    private static final String[] IMAGE_NAMES =
        {"frog1.gif","frog2.gif","frog3.gif","frog4.gif","frog5.gif"};

    public FrogImage() {
        super(IMAGE_NAMES);
    }

    public void jumpBy(int step) {
        AnimationClip clip = new JumpingFrogClip(this,step*50);
        clip.play(true);
        try {
            Thread.sleep(50);
        } catch (InterruptedException ie) {
        }
    }

    public static void main(String[] args) {
        Frame f = new Frame();
        f.setSize(1100,500);
        Panel p = new javalib.gui.DoubleBufferedPanel();
        p.setLayout(null);
        f.add(p,"Center");
        FrogImage frog = new FrogImage();
        int base = 50;
        frog.setLocation(base,200);
        p.add(frog);
        BulbImage[] bulbs = new BulbImage[16];
        for (int i=0; i<bulbs.length; i++) {
            BulbImage bulb = new BulbImage();
            bulbs[i]=bulb;
            bulb.setLocation(base+100+i*50,252);
            p.add(bulb);
            Label l = new Label(String.valueOf(i+1));
            l.setFont(new Font("Dialog",Font.BOLD,15));
            l.setSize(50,30);
            l.setLocation(base+(i>8 ? 102 : 106)+i*50,282);
            p.add(l);
        }
        f.setVisible(true);
        for (int step=1; step<=bulbs.length; step++) {
            frog.setLocation(base,200);
            try {
                Thread.sleep(1000);
            } catch (InterruptedException ie) {
            }
            for (int i=0; i+step<=bulbs.length;) {
                frog.jumpBy(step);
                i+=step;
                bulbs[i-1].setOn(!bulbs[i-1].isOn());
            }
            frog.jumpBy(step);
        }
    }

}

class JumpingFrogClip extends JumpClip {

    private FrogImage frog;

    public JumpingFrogClip(FrogImage frog, int distance) {
        super(frog,
            new Point(frog.getLocation()),
            new Point(frog.getLocation().x+distance,frog.getLocation().y),
            50,10+distance/40);
        setPreferredFrameDelay(20);
        this.frog = frog;
        frog.setState(1);
    }

    public void update() {
        if (step > 0.75*ticks) {
            frog.setState(4);
        } else if (step > 0.6*ticks) {
            frog.setState(3);
        } else if (step > 0.4*ticks) {
            frog.setState(2);
        }
        super.update();
    }

    public void stop() {
        super.stop();
        frog.setState(0);
        frog.repaint();
    }

}