/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.util.*;
import javalib.gui.*;
import java.awt.event.*;

public class FrogRiddleVisualization {

    static FrogRiddleFrame visualization;

    static {
        visualization =
            new FrogRiddleFrame();
    }

    public static void visualizeFrogCreation(Frog frog) {
        visualization.createFrog(frog);
    }

    public static void visualizeBulbCreation(Bulb bulb) {
        visualization.createBulb(bulb);
    }

    public static void visualizeFrogJump(Frog frog, int steps) {
        visualization.visualizeFrogJump(frog,steps);
    }

    public static void visualizeFrogPositionChange(Frog frog) {
        visualization.visualizeFrogPositionChange(frog);
    }

    public static void visualizeBulbStateChange(Bulb bulb) {
        visualization.visualizeBulbStateChange(bulb);
    }
}

class FrogRiddleFrame extends Frame {

    private int base = 50;
    private int offset = 50;
    private int stepSize = 50;
    private int bulbsY = 252;
    private int labelsY = 282;
    private int frogY = 200;

    private int numberOfBulbs;

    Panel display;
    Hashtable figures;

    public FrogRiddleFrame() {
        setSize(800,500);
        display = new DoubleBufferedPanel();
        display.setLayout(null);
        add(display, BorderLayout.CENTER);
        Panel bottomPanel = new Panel();
        add(bottomPanel, BorderLayout.SOUTH);
        Button exitButton = new Button("Exit");
        bottomPanel.add(exitButton);
        exitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                System.exit(0);
            }
        });
        addWindowListener(new WindowAdapter() {
            public void windowClosing() {
                System.exit(0);
            }
        });
        figures = new Hashtable();
        launcher = new LauncherThread();
        launcher.start();
    }

    private long priviousCallTime;

    public void createBulb(Bulb bulb) {
        numberOfBulbs++;
        BulbImage image = new BulbImage();
        int x = base+offset+numberOfBulbs*stepSize;
        int y = bulbsY;
        if (x+100>getSize().width) {
            setSize(getSize().width+200,getSize().height);
        }
        image.setLocation(x,y);
        display.add(image);

        Label l = new Label(String.valueOf(numberOfBulbs));
        l.setFont(new Font("Dialog",Font.BOLD,15));
        l.setSize(50,30);
        l.setLocation(base+offset+(numberOfBulbs>9 ? 2 : 6)+
            numberOfBulbs*stepSize,labelsY);
        display.add(l);

        validate();
        image.repaint();
        figures.put(bulb,image);
    }

    public void createFrog(Frog frog) {
        FrogImage image = new FrogImage();
        int x = base;
        int y = frogY;
        image.setLocation(x,y);
        display.add(image);
        figures.put(frog,image);
    }

    public void visualizeFrogJump(Frog frog, int steps) {
        if (!isVisible()) {
            launch();
            try {
                Thread.sleep(1000);
            } catch(InterruptedException ie) {}
        }

        FrogImage image = (FrogImage)figures.get(frog);
        if (image==null) {
            return;
        }
        if (steps<=0) {
            int position = frog.getPosition()+steps;
            image.setLocation(base+position*stepSize,frogY);
            repaint();
            return;
        }
        image.jumpBy(steps);
    }

    public void visualizeFrogPositionChange(Frog frog) {
        FrogImage image = (FrogImage)figures.get(frog);
        if (image==null) {
            return;
        }
        int position = frog.getPosition();
        image.setLocation(base+position*stepSize,frogY);
        repaint();
    }

    public void visualizeBulbStateChange(Bulb bulb) {
        BulbImage image = (BulbImage)figures.get(bulb);
        if (image==null) {
            return;
        }
        image.setOn(bulb.isOn());
    }

    private LauncherThread launcher;

    protected synchronized void launch() {
        if (isVisible()) {
            return;
        }
        setVisible(true);
        launcher.interrupt();
    }

    class LauncherThread extends Thread {

        public void run() {
            try {
                Thread.sleep(4000);
            } catch (InterruptedException ie) {
                return;
            }
            launch();
        }
    }
}



