/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.awt.image.*;

public class ImageSetComponent extends Component {

    private Image[] images;

    protected int state = 0;

    public ImageSetComponent(String[] names) {
        loadImages(names);
        preferredSize = computePreferredSize();
        Dimension size = getPreferredSize();
        setSize(size.width,size.height);
    }

    public void paint(Graphics g) {
        Image image = images[state];
        int x = (getSize().width-image.getWidth(this))/2;
        int y = (getSize().height-image.getHeight(this))/2;
        g.drawImage(image,x,y,this);
    }

    protected void setState(int state) {
        this.state = state;
    }

    private Dimension preferredSize;

    protected Dimension computePreferredSize() {
        int width = 0;
        int height =0;
        for (int i=0; i<images.length; i++) {
            width = Math.max(width,images[i].getWidth(this));
            height = Math.max(height,images[i].getHeight(this));
        }
        return new Dimension(width,height);
    }

    public Dimension getPreferredSize() {
        return preferredSize;
    }

    public Dimension getMinimalSize() {
        return getPreferredSize();
    }

    protected void loadImages(String[] names) {
        images = new Image[names.length];
        MediaTracker tracker = new MediaTracker(this);
        for (int i=0; i<names.length; i++) {
            images[i]=getImage("gifs/"+names[i]);
            tracker.addImage(images[i],0);
        }
        try {
            tracker.waitForID(0);
        } catch (InterruptedException ie) {
        }
    }

    protected Image getImage(String name) {
        try {
            java.net.URL resource =  getClass().getResource(name);
            ImageProducer producer = (ImageProducer) resource.getContent();
            return getToolkit().createImage(producer);
        } catch (Exception e) {
            return null;
        }
    }

}