/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;

public class DiskIcon extends Component {

    // The pile to which the chip belongs
    private int pile;
    
    // The width of the disk in units
    private int units;

    private static final int BASE_WIDTH = 24;
    private static final int UNIT_WIDTH = 8;
    private static final int DEFAULT_HEIGHT = 20;

    /**
     * Constructs a new chip.
     */
    public DiskIcon(int units, Color color) {
        this.units = units;
        setSize(getPreferredSize().width,getPreferredSize().height);
        setBackground(color);
    }

    /**
     * Return the center point of this component.
     */
    public Point getCenter() {
        Point center = new Point(getLocation());
        center.translate(getSize().width/2,getSize().height/2);
        return center;
    }

    /**
     * Paint this disk.
     */
    public void paint(Graphics g) {
        int w = getSize().width-1;
        int h = getSize().height-1;
        g.setColor(getBackground());
        g.fillRoundRect(0,0,w,h,8,8);
        g.setColor(getForeground());
        g.drawRoundRect(0,0,w,h,8,8);
    }

    /**
     * Returns the preferred size of this component.
     */
    public Dimension getPreferredSize() {
        Dimension d = new Dimension(BASE_WIDTH+units*UNIT_WIDTH,DEFAULT_HEIGHT);
        return d;
    }

    /**
     * Returns the minimal size of this component.
     */
    public Dimension getMinimalSize() {
        return getPreferredSize();
    }

}