/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.util.*;

public class Pile {

    private Stack disks;
    
    /**
     * Creates a new empty pile.
     */
    public Pile() {
        disks = new Stack();
        HanoiVisualization.visualizePileCreation(this);
    }

    public final int getNumberOfDisks() {
        return disks.size();
    }

    public Disk removeDisk() {
        try {
            Disk disk = (Disk)disks.peek();
            HanoiVisualization.visualizeRemoveDisk(this, disk);
            disks.pop();
            return disk;
        } catch (EmptyStackException e) {
            throw new IllegalOperationException(
                "Attempt to remove a disk from an empty pile.");
        }
    }
    
    public void addDisk(Disk disk) {
        if (!disks.isEmpty() && topDisk().getSize()<disk.getSize()) {
            throw new IllegalOperationException(
                "You are not allowed to put a bigger disk on a smaller one.");
        }
        HanoiVisualization.visualizeAddDisk(this, disk);
        disks.push(disk);
    }
    
    public Disk topDisk() {
        return (Disk)disks.peek();
    }
    
    public boolean isEmpty() {
        return disks.isEmpty();
    }

}