/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import jclip.*;
import java.awt.*;

public class RobotArmMotionClip extends
    AnimationClip {

    private Point from,to;
    private float x1,y1;
    private float x2,y2;
    private float portion;
    private float step;
    private RobotArmFigure arm;
    private int xOffset, yOffset;
    private Component display;
    private Component load;

    public RobotArmMotionClip(
        Component display,
        RobotArmFigure arm,
        Point from, Point to,
        int ticks) {

        this.display=display;
        this.arm = arm;
        this.from = new Point(from);
        this.to = new Point(to);
        step = 1.0f/ticks;
        x2=to.x;
        y2=to.y;
    }

    public void setLoad(Component load) {
        this.load=load;
    }

    public void init() {
        x1= from.x;
        y1= from.y;
        portion = 0.0f;
        arm.setHeadLocation(from);
        display.repaint();
        if (load!=null) {
            xOffset=-load.getSize().width/2;
            yOffset=-load.getSize().height;
        }
    }

    public void stop() {
        super.stop();
        arm.setHeadLocation(to);
        display.repaint();
    }

    public void update() {
        portion+=step;
        if (portion>=1.0) {
            endClip();
            return;
        }
        float x = x1 + (x2-x1)*portion;
        float y = y1 + (y2-y1)*portion;
        Point head = new Point((int)x,(int)y);
        arm.setHeadLocation(head);
        if (load!=null) {
            load.setLocation(head.x+xOffset,head.y+yOffset);
            load.repaint();
        }
        display.repaint();
    }

}