/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import jclip.*;
import java.awt.*;

public class RobotFingerMovementClip extends
    AnimationClip {

    private int initialSpacing;
    private int finalSpacing;
    private float portion;
    private float step;
    private RobotArmFigure arm;
    private Component display;

    public RobotFingerMovementClip(Component display,RobotArmFigure arm,
        int finalSpacing, int ticks) {

        this(display,arm,arm.getFingerSpacing(),finalSpacing,ticks);
    }

    public RobotFingerMovementClip(Component display,RobotArmFigure arm,
        int initialSpacing, int finalSpacing,
        int ticks) {

        this.display=display;
        this.arm = arm;
        this.initialSpacing = initialSpacing;
        this.finalSpacing = finalSpacing;
        step = 1.0f/ticks;
    }

    public void init() {
        super.init();
        arm.setFingerSpacing(initialSpacing);
        display.repaint();
        portion = 0.0f;
    }

    public void stop() {
        super.stop();
        arm.setFingerSpacing(finalSpacing);
        display.repaint();
    }

    public void update() {
        portion+=step;
        if (portion>=1.0) {
            endClip();
            return;
        }
        int spacing = (int)(initialSpacing +
            (finalSpacing-initialSpacing)*portion);
        arm.setFingerSpacing(spacing);
        display.repaint();
    }

}