/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.util.*;
import jclip.*;
import clips.motion.*;

public class NimGameVisualizationPanel extends DoubleBufferedPanel {

    Hashtable figures;
    Hashtable trashes;
    Vector heaps;

    private int ground;
    private Dimension chipSize = new Dimension(30,13);
    private int hspacing = 15;
    private int vspacing = 1;
    private int numberOfRobots;

    public NimGameVisualizationPanel() {
        figures = new Hashtable();
        trashes = new Hashtable();
        heaps = new Vector();
    }

    public void init() {
        setLayout(null);
        ground = getSize().height-100;
    }

    public void visualizeHeapCreation(NimHeap heap) {
        Stack heapImage = createHeapImage(heap);
        Rectangle bounds = computeChipBounds(heaps.size()-1,1);
        heaps.addElement(heapImage);
        int x = bounds.x+hspacing;
        int y = ground;
        for (int i=0; i<heapImage.size(); i++) {
            NimChipIcon icon = (NimChipIcon)heapImage.elementAt(i);
            icon.setBounds(x,y,chipSize.width,chipSize.height);
            add(icon);
        }
        reorderChips();
    }

    private Stack createHeapImage(NimHeap heap) {
        Stack heapImage = new Stack();
        for (int i=0; i<heap.getSize(); i++) {
            NimChipIcon icon = new NimChipIcon();
            heapImage.push(icon);
        }
        return heapImage;
    }

    protected void reorderChips() {
        for (int i=0; i<heaps.size(); i++) {
            Stack heap = (Stack)heaps.elementAt(i);
            for (int j=0; j<heap.size(); j++) {
                NimChipIcon icon = (NimChipIcon)heap.elementAt(j);
                icon.setBounds(computeChipBounds(i,j));
            }
        }
    }

    protected Rectangle computeChipBounds(int pile , int i) {
        int w = chipSize.width+hspacing;
        int x = (getSize().width-heaps.size()*w)/2+
            w*pile;
        int y = ground-(chipSize.height+vspacing)*i;
        return new Rectangle(x,y,chipSize.width,chipSize.height);
    }

    public void visualizeRobotTakesChip(
        NimPlayerRobot robot, int pile) {

        NimChipIcon icon = (NimChipIcon)
            ((Stack)heaps.elementAt(pile)).pop();
        RobotArmFigure arm = (RobotArmFigure)
            figures.get(robot);
        Point origin = arm.getHeadLocation();
        Point target = icon.getCenter();
        target.translate(0,-100); // temp
        TrashFigure trash = (TrashFigure)
            trashes.get(robot);

        RobotArmMotionClip moveAboveChip =
            new RobotArmMotionClip(this,arm,origin,target,30);
        RobotFingerMovementClip openHand =
            new RobotFingerMovementClip(this,arm,
                icon.getSize().width+6,10);
        origin = target;
        origin.translate(0,10);
        target = icon.getCenter();
        RobotArmMotionClip goDown =
            new RobotArmMotionClip(this,arm,origin,target,30);
        RobotArmMotionClip goUp =
            new RobotArmMotionClip(this,arm,target,origin,30);
        goUp.setLoad(icon);
        target = trash.above();
        RobotArmMotionClip moveAboveTrash =
            new RobotArmMotionClip(this,arm,origin,target,30);
        moveAboveTrash.setLoad(icon);
        RobotFingerMovementClip openWide =
            new RobotFingerMovementClip(this,arm,
                icon.getSize().width+6,
                icon.getSize().width+12,10);
        AnimationClip closeHand =
            new RobotFingerMovementClip(this,arm,
                icon.getSize().width+12,
                20,10);
        AnimationClip fall =
            new FreeFallClip(icon,target,trash.getLocation(),25);
        AnimationClip parallel =
            new AnimationClipUnion(closeHand,fall);
        AnimationClip drop =
            new AnimationClipSequence(openWide,parallel);
        AnimationClip[] clips =
            {moveAboveChip,openHand,goDown,goUp,
             moveAboveTrash,parallel};
        AnimationClip clip =
            new AnimationClipSequence(clips);
        clip.setPreferredFrameDelay(35);
        clip.play(true);
        try {
            Thread.sleep(500);
        } catch (InterruptedException ie) {}
    }

    public void visualizeRobotCreation(NimPlayerRobot robot) {
        RobotArmFigure figure = new RobotArmFigure(getSize().height/2,30);
        numberOfRobots++;
        int x = numberOfRobots%2!=0 ? getSize().width-100 : 100;
        int dx = numberOfRobots==1 ? -1 : 1;
        int y = ground;
        figure.setLocation(x,y);
        figures.put(robot,figure);
        figure.setHeadLocation(new Point(x+dx*200,y-200));

        TrashFigure trash = new TrashFigure();
        x = x+dx*100;
        trash.setLocation(x,ground);
        trashes.put(robot,trash);
    }

    public void paint(Graphics g) {
        for (Enumeration list = figures.elements(); list.hasMoreElements();) {
            Figure figure = (Figure)list.nextElement();
            figure.draw(g);
        }
        super.paint(g);
    }
}