/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;

public class RobotArmJoint extends Figure {

    // The length of the joint
    private int size;

    // The thickness of the joint
    private int thickness;

    // The angle of the joint in radians
    private double teta;

    public RobotArmJoint(int size, int thickness) {
        this.size = size;
        this.thickness = thickness;
        setInteriorColor(Color.lightGray);
        setBorderColor(Color.black);
    }

    protected Polygon computeBoundery() {
        Point base = new Point(x,y);
        int alpha = radiansToDegrees(teta);
        Point p1 = relativePoint(base,alpha-90,thickness/2);
        Point p2 = relativePoint(p1,alpha,size);
        Point p3 = relativePoint(p2,alpha+90,thickness);
        Point p4 = relativePoint(base,alpha+90,thickness/2);
        int[] xs = {p1.x,p2.x,p3.x,p4.x};
        int[] ys = {p1.y,p2.y,p3.y,p4.y};
        return new Polygon(xs,ys,4);
    }

    public Point getEndPoint() {
        Point base = new Point(x,y);
        int alpha = radiansToDegrees(teta);
        return relativePoint(base,alpha,size);
    }

    private Point relativePoint(Point p, double teta, int size) {
        int x = (int)(p.x + size*Math.cos(teta));
        int y = (int)(p.y - size*Math.sin(teta));
        return new Point(x,y);
    }

    private Point relativePoint(Point p, int alpha, int length) {
        return relativePoint(p,Math.PI*alpha/180,length);
    }

    private int radiansToDegrees(double teta) {
        return (int)(teta*180/Math.PI);
    }

    private double degreesToRadians(int alpha) {
        return alpha*Math.PI/180;
    }

    public void draw(Graphics g) {
        Polygon boundery = computeBoundery();
        g.setColor(getInteriorColor());
        g.fillPolygon(boundery);
        g.setColor(getBorderColor());
        g.drawPolygon(boundery);
    }

    public void setAngleRadians(double teta) {
        this.teta=teta;
    }

    public final double getAngleRadians() {
        return teta;
    }

    public final void setAngleDegrees(int alpha) {
        setAngleRadians(Math.PI*alpha/180);
    }

    public final int getAngleDegrees() {
        return (int)(teta*180/Math.PI);
    }

    public final int getSize() {
        return size;
    }

    public final int getThickness() {
        return thickness;
    }
}