/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.awt.event.*;

/**
 * An object that receive input from the user.
 *
 * Shmulik London
 */
public class InputRequestor extends Frame {

    private static final Color disabledColor = Color.lightGray;
    private static final Color enabledColor = Color.white;

    // The text field into which the user enter the input
    private TextField input;

    // A Label describing the input request
    private Label label;

    private String inputString;

    /**
     * Constructs a new InputRequestor.
     */
    public InputRequestor() {
        super("Input Requestor");
        Panel rootPane = new Panel();
        add(rootPane, BorderLayout.CENTER);
        rootPane.setLayout(new FlowLayout(FlowLayout.LEFT));
        Panel mainPanel = new Panel();
        rootPane.add(mainPanel);
        mainPanel.setLayout(new GridLayout(2,1,10,10));
        Font font = new Font("Dialog", Font.BOLD, 16);
        label = new Label("                                     ");
        label.setFont(font);
        mainPanel.add(label);
        input = new TextField(40);
        input.setFont(font);
        mainPanel.add(input);
        pack();
        setEnabledInput(false);
        ActionListener listener = new InputHandler();
        input.addActionListener(listener);
        setVisible(true);
    }

    // Prepare the window for receiving input
    // The window moves to the center of the screen and becomes
    // enabled.
    private void setEnabledInput(boolean enabled) {
        if (enabled) {
            setBackground(enabledColor);
            Dimension screenSize = getToolkit().getScreenSize();
            int x = (screenSize.width-getSize().width)/2;
            int y = (screenSize.height-getSize().height)/2;
            setLocation(x,y);
            input.setEnabled(true);
            input.setEditable(true);
            label.setVisible(true);
            requestFocus();
            input.requestFocus();
        }
        else {
            setBackground(disabledColor);
            input.setEnabled(false);
            input.setEditable(false);
            input.setText("");
            label.setVisible(false);
            setLocation(0,0);
            repaint();
        }
    }

    /**
     * Requests an integer value from the user.
     */
    public int requestInt(String requestMessage) {
        while (true) {
            String reply = requestString(requestMessage);
            try {
                return Integer.parseInt(reply);
            } catch (Exception e) {
            }
        }
    }

    /**
     * Requests an integer value from the user.
     */
    public double requestDouble(String requestMessage) {
        while (true) {
            try {
                String reply = requestString(requestMessage);
                return new Double(reply).doubleValue();
            } catch (Exception e) {
            }
        }
    }

    /**
     * Requests a float value from the user.
     */
    public float requestFloat(String requestMessage) {
        return (float)requestDouble(requestMessage);
    }

    /**
     * Request a string from the user.
     */
    public synchronized String requestString(String requestMessage) {
        label.setText(requestMessage);
        label.setVisible(true);
        pack();
        setEnabledInput(true);
        try {
            wait();
        } catch (InterruptedException ie) {}
        setEnabledInput(false);
        return inputString;
    }

    private synchronized void inputReceived(String input) {
        inputString = input;
        notifyAll();
    }

    class InputHandler implements ActionListener {

        public void actionPerformed(ActionEvent event) {
            String text = input.getText().trim();
            if (text.equals("")) {
                return;
            }
            inputReceived(text);
        }
    }

}