/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.io.*;
import java.util.*;

/**
 * A TextDocument gives a simple way for reading text files line by line.
 */
public class TextDocument {

    private Vector text;
    private int index;

    /**
     * Creates a new text document from a given source file.
     * @exception TextException The text could not be found.
     */
    public TextDocument(String source) {
        try {
            text = loadText(source);
        } catch (IOException ioe) {
            throw new TextException("Couldn't find text");
        }
    }

    private Vector loadText(String source)
        throws IOException {
            BufferedReader reader =
                new BufferedReader(new FileReader(source));
            String line;
            Vector lines = new Vector();
            while ((line=reader.readLine())!=null) {
                lines.addElement(line);
            }
            return lines;
    }

    /**
     * Checks if there are more lines to be read from the text.
     * @return <TT>true</TT> if there are more lines to be read
     * from the text.
     */
    public boolean hasMoreLines() {
        return (index<text.size());
    }

    /**
     * Reads the next line from the text.
     * @return The next line in the text.
     */
    public String nextLine() {
        try {
            String line = (String)text.elementAt(index);
            index++;
            return line;
        } catch (ArrayIndexOutOfBoundsException aioobe) {
            throw new TextException("Attempt to read beyond the "+
                " document last line.");
        }
    }
}
