/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.util.*;
import java.io.Serializable;
import javalib.gui.DoubleBufferedPanel;

/**
 * A Logo plane in which turtles live and draw figures.
 *
 * @author Shmulik London
 */
public class LogoPlane extends DoubleBufferedPanel {

    // Records all the lines that where drawn by the turtles
    private Vector drawing;

    /**
     * Constructs a new LogoPlane.
     */
    public LogoPlane() {
        drawing = new Vector();
        setLayout(null);
    }

    /**
     * Adds a line segment.
     * @param from,till The starting and ending points of
     * the line segment.
     * @param pen The color of the line segment.
     */
    public void addLineSegment(Point from, Point till, Color pen) {
        drawing.addElement(new LineSegment(from,till,pen));
        repaint();
    }

    /**
     * Adds a line segment.
     * @param from,till The starting and ending points of
     * the line segment.
     */
    public void addLineSegment(Point from, Point till) {
        addLineSegment(from, till, Color.black);
    }

    /**
     * Paints this component.
     */
    public void paint(Graphics g) {
        for (int i=0; i<drawing.size(); i++) {
            LineSegment segment =
                (LineSegment)drawing.elementAt(i);
            g.setColor(segment.color);
            g.drawLine(segment.from.x,segment.from.y,
                       segment.till.x,segment.till.y);
        }
        super.paint(g);
    }

    /**
     * Adds a new turtle to this plane.
     */
    public void addTurtle(TurtleImage image) {
        image.setLocation(getSize().width/2,getSize().height/2);
        add(image);
    }
}

// This class is used to hold the information about
// a single line segment
class LineSegment implements Serializable {

    /**
     * Constructs a LineSegement.
     * @param from,till The starting and ending points of
     * the line segment.
     * @param color The color of the line segment.
     */
    public LineSegment(Point from, Point till, Color color) {
        this.from = from;
        this.till = till;
        this.color = color;
    }

    /**
     * Constructs a LineSegement. The default color of the
     * segment is black.
     * @param from,till The starting and ending points of
     * the line segment.
     */
    public LineSegment(Point from, Point till) {
        this(from,till,Color.black);
    }

    /**
     * The starting and ending points of the segment.
     */
    public Point from,till;

    /**
     * The color of the line segment.
     */
    public Color color;
}