/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * A turtle of LOGO.
 * @author Shmulik London
 */
public class Turtle {

    // The direction of the turtle in degrees
    private double alpha = 90;

    // True iff the tail of the turtle is down
    private boolean tailDown;

    // The image of this turtle
    private TurtleImage image;

    // The delay after each operation of the turtle
    private int delay = 100;

    /**
     * Constructs a new turtle. <BR>
     * Note: In this version, the construction of the first
     * turtle has a side effect of opening a new window and
     * placing the turtle onto it. Any additional created
     * turtles will be placed on the same window.
     */
    public Turtle() {
        image = new TurtleImage();
        image.setAngle(degreesToRadians(alpha));
    }

    /**
     * Advances the turtle forwards by a given number of
     * units. The turtle advances in the direction it is
     * currently looking at.
     * @param units The number of units the turtle should
     * advance by.
     */
    public void moveForward(double units) {
        double dx = units*Math.cos(degreesToRadians(alpha));
        double dy = -units*Math.sin(degreesToRadians(alpha));
        image.moveBy((int)dx,(int)dy,tailDown);
        delay();
    }

    /**
     * Moves the turtle backwards by a given number of units.
     * @param units The number of units the turtle shuold
     * move backwards by.
     * @see #moveForward(double)
     */
    public void moveBackward(double units) {
        moveForward(-units);
    }

    /**
     * Turns the turtle counter-clockwise. The turtle
     * changes the direction it is looking at by the given
     * number of degrees.
     * @param degrees The angle by which the turtle should
     * turn
     */
    public void turnLeft(int degrees) {
        alpha+=degrees;
        image.setAngle(degreesToRadians(alpha));
        delay();
    }

    /**
     * Turns the turtle clockwise. The turtle changes the
     * direction it is looking at by the given number of
     * degrees.
     * @param degrees The angle by which the turtle should
     * turn
     */
    public void turnRight(int degrees) {
        turnLeft(-degrees);
    }

    /**
     * Lowers the tail of the turtle. Consequent movements
     * of the turtle will leave behind it a line.
     */
    public void tailDown() {
        tailDown = true;
    }

    /**
     * Raises the tail of the turtle. Consequent movements
     * of the turtle will not leave any mark behind it.
     */
    public void tailUp() {
        tailDown = false;
    }

    /**
     * Hides the turtle.
     */
    public void hide() {
        image.setVisible(false);
    }

    /**
     * Shows the turtle.
     */
    public void show() {
        image.setVisible(true);
    }

    // Delay the execution of consequent instructions
    private void delay() {
        if (delay==0) {
            return;
        }
        try {
            Thread.sleep(delay);
        } catch (InterruptedException ie) {
            // Ignore
        }
    }

    // Converts degrees to radians
    private double degreesToRadians(double degrees) {
        return (Math.PI*degrees/180);
    }
}

