package electronics.ui;

import java.awt.*;

public class ConnectionPainter {
 
    // The color of the line representing the connection
    private Color color = Color.black;
    
    /**
     * Sets the color used to draw the line representing the 
     * connection.
     */
    public void setColor(Color color) {
        this.color = color;
    }
    
    /** 
     * Return the color used to draw the line representing the 
     * connection.
     */
    public final Color getColor() {
        return color;
    }
    
    /**
     * Paint a connection between two points.
     */
    public void paintConnection(Graphics g, Point a, Point b) {
        Point m = new Point(a.x+(b.x-a.x)/2, a.y+(b.y-a.y)/2);
        Color prev = g.getColor();
        g.setColor(color);
        if (b.x > a.x) {
            drawHLine(g, a.y, a.x, m.x);
            drawVLine(g, m.x, a.y, b.y);
            drawHLine(g, b.y, m.x, b.x);
        }
        else {
            int detour = 20;
            drawHLine(g, a.y, a.x, a.x+detour);
            drawVLine(g, a.x+detour, a.y, m.y);
            drawHLine(g, m.y, a.x+detour, b.x-detour);
            drawVLine(g, b.x-detour, m.y, b.y);
            drawHLine(g, b.y, b.x-detour, b.x);
        }
    }
    
    private void drawVLine(Graphics g, int x, int y1, int y2) {
        g.drawLine(x, y1, x, y2);
        g.drawLine(x-1, y1, x-1, y2);
    }
    
    private void drawHLine(Graphics g, int y, int x1, int x2) {
        g.drawLine(x1, y, x2, y);
        g.drawLine(x1, y-1, x2, y-1);
    }
}