// Copyright (c) 1995, Shmulik London londons@cs.huji.ac.il
// Permission granted for non-comertial use provided this copyright
// note remains at the top of the source file.
// Commertial use may be granted upon e-mail request.
// No garentee is made for the correctness of this code.
// We will not be liable for any damage resuting from using this code
// or any derived code.

package electronics.ui;

import java.awt.*;
import java.util.*;

/**
 * A Panel that uses dubble-buffering to avoid flashing.
 */
public class DoubleBufferedPanel extends Panel {

    private Graphics bufferGraphics;
    private Dimension bufferDimension;
    private transient Image bufferImage;
    private transient Image backgroundImage;
    private boolean backgroundChanged = true;

    /**
     * Updates the display of the component.
     */
    public void update(Graphics g) {
        if ( (bufferGraphics == null)
             || !(bufferDimension.equals(getSize()))) {
            bufferDimension = getSize();
            bufferImage = createImage(getSize().width, getSize().height);
            bufferGraphics = bufferImage.getGraphics();
            backgroundImage = createImage(getSize().width, getSize().height);
            paintBackground(backgroundImage.getGraphics());
        }
        if (backgroundChanged) {
            Graphics backgroundGraphics = backgroundImage.getGraphics();
            backgroundGraphics.clearRect(0,0,getSize().width,getSize().height);
            paintBackground(backgroundGraphics);
        }
        bufferGraphics.drawImage(backgroundImage,0,0,this);
        paint(bufferGraphics);
        g.drawImage(bufferImage, 0, 0, this);
    }

    /**
     * Updates the background of the component.
     * This method should be called each time the
     * background painting has changed.
     */
    protected void updateBackground() {
        backgroundChanged=true;
        repaint();
    }

    /**
     * Paints a constant background for the component.
     * Use <TT>updateBackground()</TT> when the background
     * has changed.
     */
    public void paintBackground(Graphics g) {
    }
}
