package electronics.ui;

import java.awt.*;

/**
 * A graphical component that represents an electronic gate. This class is a super
 * class for all concrete electronic gate figures.
 * 
 * @author <A Href=mailto:londons@cs.huji.ac.il>Shmulik London</A>
 */
public abstract class ElectronicGateFigure extends Component {
  
    // The number of input legs of the gate this figure represents
    private int numberOfInputs;

    /**
     * Constructs an ElectronicGateFigure with a given number of input legs.
     * @param numberOfInputs The nubmer of input legs of this gate.
     */
    protected ElectronicGateFigure(int numberOfInputs) {
        this.numberOfInputs = numberOfInputs;
        setBackground(Color.lightGray);
        setForeground(Color.black);
    }

    /**
     * Returns the nubmer of input legs of the gate this figure represents.
     */
    public int getNumberOfInputs() {
        return numberOfInputs;
    }

    /**
     * Paints the figure of the gate.
     */
    public abstract void paint(Graphics g);

    /**
     * Returns the preferred size of this component.
     */
    public abstract Dimension getPreferredSize();

    /**
     * Returns the location of the given input connection point of the gate
     * in its container coordinate system.
     * @param i The index of the input point.
     */
    public Point getInputLocation(int i) {
        int x = getLocation().x;
        int y;
        int margin = 8;
        if (numberOfInputs > 1) {
            int step = (getSize().height-2*margin)/(numberOfInputs-1);
            y = getLocation().y+margin+step*i;
        }
        else {
            y = getLocation().y+getSize().height/2;
        }
        return new Point(x,y);
    }

    /**
     * Returns the location of the output connection point of the gate
     * in its container coordinate system.
     */
    public Point getOutputLocation() {
        int x = getLocation().x+getSize().width;
        int y = getLocation().y+getSize().height/2;
        return new Point(x,y); 
    }

}