package electronics.ui;

import electronics.*;

/**
 * A factory for electronic gates. An ElectronicGateFactory object can produce 
 * gates according to their names.
 *
 * @author <A Href=mailto:londons@cs.huji.ac.il>Shmulik London</A>
 */
public class ElectronicGateFigureFactory {

    /**
     * Creates an electronic gate figure that correspond to the given name.
     * @param type The name of the electronic gate type
     * (OR, AND, NOT, NOR, NAND, XOR).
     * @exception UnfamiliarGateException The specified gate name 
     * is not known.
     */
    public ElectronicGateFigure createElectronicGateFigure(String type) 
	    throws UnfamiliarGateException {

        type = type.trim().toUpperCase();

        if (type.equals("OR")) {
            return new OrGateFigure();
        }
        if (type.equals("NOR")) {
            return new NorGateFigure();
        }
        if (type.equals("AND")) {
            return new AndGateFigure();
        }
        if (type.equals("NAND")) {
            return new NandGateFigure();
        }
        if (type.equals("NOT")) {
            return new NotGateFigure();
        }
        if (type.equals("XOR")) {
            return new XorGateFigure();
        }
        throw new UnfamiliarGateException();
    }
}