package electronics.ui;

import java.awt.*;
import electronics.*;

/**
 * A graphical component that represents an or gate. 
 * 
 * @author <A Href=mailto:londons@cs.huji.ac.il>Shmulik London</A>
 */
public class NotGateFigure extends ElectronicGateFigure {
  
    private static final int CIRCLE_SIZE = 8;
    
    /**
     * Constructs a new OrGateFigure.
     */
    protected NotGateFigure() {
        super(1); // temp
    }

    /**
     * Paints the figure of the gate.
     */
    public void paint(Graphics g) {
        int hMargin = 5;
        int x = hMargin;
        int y = 0;
        int width = getSize().width-1-hMargin*2-CIRCLE_SIZE;
        int height = getSize().height-1;
        g.setColor(getBackground());
        Point a = new Point(hMargin,0);
        Point b = new Point(hMargin+width,height/2);
        Point c = new Point(hMargin,height);
        int[] xs = {a.x, b.x, c.x};
        int[] ys = {a.y, b.y, c.y};
        g.fillPolygon(xs, ys, 3);
        g.fillOval(getSize().width-hMargin-CIRCLE_SIZE,
            (height-CIRCLE_SIZE)/2, CIRCLE_SIZE, CIRCLE_SIZE);
        g.setColor(getForeground());
        g.drawPolygon(xs, ys, 3);
        g.drawOval(getSize().width-hMargin-CIRCLE_SIZE,
            (height-CIRCLE_SIZE)/2, CIRCLE_SIZE, CIRCLE_SIZE);
            
        y = getInputLocation(0).y-getLocation().y;
        g.drawLine(0, y, hMargin, y);
        g.drawLine(0, y-1, hMargin, y-1);
        y = getOutputLocation().y-getLocation().y;
        g.drawLine(getSize().width, y, getSize().width-hMargin, y);
        g.drawLine(getSize().width, y-1, getSize().width-hMargin, y-1);
    }

    /**
     * Returns the preferred size of this component.
     */
    public Dimension getPreferredSize() {
        return new Dimension(25+CIRCLE_SIZE,25);
    }

}