/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * A clock. A Clock object holds the time at a given place and time.
 * it is constructed with a given initial time, and has methods
 * that advances the time.
 * <P>
 * <B>Example of use:</B>
 * <PRE>
 * Clock clock = new Clock(1, 29, 0);
 * System.out.println("The current time is: "+clock);
 * clock.minuteElapsed();
 * System.out.println(clock);
 * int currentMinute = clock.getMinute();
 * System.out.println("Current minute is "+currentMinute);
 * System.out.println("Changing to winter clock");
 * clock.setTime(clock.getHour()-1, clock.getMinute(), clock.getSecond());
 * System.out.println("New time: "+clock);
 * </PRE>
 *
 * @author Shmulik London
 */
public class Clock {

    // Hold the time displayed by the clock
    private int hour, minute, second;

    /**
     * Constructs a new clock initialized with a given time.
	 * If the given arguments doesn't represent a valid date
	 * an IllegalArgumentException is thrown.
     * @param hour, minute, second Specify the initial time.
	 * @exception java.lang.IllegalArgumentException If the arguments
	 * doesn't represent a valid time.
     */
    public Clock(int hour, int minute, int second) {
        setTime(hour, minute, second);
    }

    /**
     * Sets the time read by the clock.
  	 * If the given arguments doesn't represent a valid date
	 * an IllegalArgumentException is thrown.
     * @param hour, minute, second The new time.
   	 * @exception java.lang.IllegalArgumentException If the arguments
	 * doesn't represent a valid time.
     */
    public void setTime(int hour, int minute, int second) {
        if (hour<0 || hour>23 ||
            minute<0 || minute>59 ||
            second<0 || second>59) {
			throw new IllegalArgumentException("Non valid time");
		}
		this.hour = hour;
		this.minute = minute;
		this.second = second;
    }

    /**
     * Return the current hour.
	 * @return Current hour.
     */
    public int getHour() {
        return hour;
    }

    /**
     * Return the current minute of the hour.
	 * @return Current minute.
     */
    public int getMinute() {
        return minute;
    }

    /**
     * Return the second of the minute.
	 * @return Current second.
     */
    public int getSecond() {
        return second;
    }

    /**
     * Advances the clock by one second.
     */
    public void secondElapsed() {
        second = second + 1;
        if (second>59) {
            second = 0;
            minuteElapsed();
        }
    }

    /**
     * Advances the clock by one minute.
     */
    public void minuteElapsed() {
        minute = minute + 1;
        if (minute > 59) {
            minute = 0;
            hourElapsed();
        }
    }

    /**
     * Advances the clock by one hour.
     */
    public void hourElapsed() {
        hour = (hour + 1) % 24;
    }

    /**
     * Return a description of the time displayed by the clock
     * in the form hh:mm:ss.
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        if (hour<10) {
            buffer.append("0");
        }
        buffer.append(hour);
        buffer.append(":");
        if (minute<10) {
            buffer.append("0");
        }
        buffer.append(minute);
        buffer.append(":");
        if (second<10) {
            buffer.append("0");
        }
        buffer.append(second);
        return buffer.toString();
    }
}
