/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import logo.*;

/**
 * A rectangle in the plane. A Rectangle object denotes a rectangle in
 * the plane whose base is parallel to the x-axis.
 *
 * @author Shmulik London.
 */
public class Rectangle {

    // The location of the top-left corner of the rectangle
    private float x,y;

    // The dimension of the rectangle
    private float width, height;

    /**
     * Constructs a new Rectangle with a given location and size.
     * @param x,y The location of the top-left corner of the rectangle.
     * @param width, height The size of the rectangle.
     */
    public Rectangle(float x, float y, float width, float height) {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
    }

    /**
     * Returns the x-coordinate of the left edge of the rectangle.
     * @returns The x-coordinate of the left edge of the rectangle.
     */
    public float getX() {
        return x;
    }

    /**
     * Returns the y-coordinate of the top edge of the rectangle.
     * @returns The y-coordinate of the top edge of the rectangle.
     */
    public float getY() {
        return y;
    }

    /**
     * Returns the width of the rectangle.
     * @returns The width of the rectangle.
     */
    public float getWidth() {
        return width;
    }

    /**
     * Returns the height of the rectangle.
     * @returns The height of the rectangle.
     */
    public float getHeight() {
        return height;
    }

    /**
     * Checks if a given point is within the bounds of the rectangle.
     * @param x,y The given point.
     * @return <TT>true</TT> if the point (x,y) is inside the bounds of the
     * rectangle, <TT>false</TT> otherwise.
     */
    public boolean contains(float x, float y) {
        return ((x >= this.x) && (x <= this.x+width) &&
                (y >= this.y) && (y <= this.y+height));
    }

    /**
     * Computes the area of the rectangle.
     * @return The area of the rectangle.
     */
    public float area() {
        return width*height;
    }

    /**
     * Computes the perimeter of this rectangle.
     * @return The perimeter of the rectangle.
     */
    public float perimeter() {
        return 2*(width+height);
    }

    /**
     * Moves the rectangle to a new location.
     * @param x,y The new left-top corner of the rectangle.
     */
    public void moveTo(float x, float y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Stretches the edges of the rectangle by a given ratio.
     * @ratio The stretching ratio.
     */
    public void stretch(float ratio) {
        width = width*ratio;
        height = height*ratio;
    }

    /**
     * Draws the rectangle.
     */
    public void draw() {
        Turtle t = new Turtle();
        t.moveTo(x,y);
        t.setAngle(0);
        t.tailDown();
        t.moveForward(width);
        t.turnRight(90);
        t.moveForward(height);
        t.turnRight(90);
        t.moveForward(width);
        t.turnRight(90);
        t.moveForward(height);
        t.hide();
    }
}