/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.util.Enumeration;

// This program writes all the solutions for the following problem:
// A riddle: In how many ways can you place 8 queens on a chess-
// board such that no one will threaten the other.
class QueensRiddle {
    
    // The size of the chessboard
    private static final int SIZE = 8;
    
    /**
     * Solves the riddle.
     */
    public static void main(String[] args) {
        
        OutputWindow output = new OutputWindow("Queens Riddle");
        
        // permutation - all the permutation on 1..SIZE
        Enumeration permutations = Permutation.allPermutations(SIZE);
        while (permutations.hasMoreElements()) {
            Permutation position = (Permutation)permutations.nextElement();
            if (isValid(position)) {
                print(position, output);
            }
        }
    }
    
    // Checks if the position is valid - if no queen threaten
    // another queen. 
    private static boolean isValid(Permutation position) {
        int n = position.order();
        for (int i=1; i<=n; i++) {
            for (int j=1; j<=n; j++) {
                if (i==j) {
                    continue;
                }
                if (Math.abs(i-j)==Math.abs(position.map(i)-position.map(j))) {
                    return false;
                }
            }
        }
        return true;
    }
    
    // prints the board position represented by a given permutation
    private static void print(Permutation position, OutputWindow output) {
        int n = position.order();
        for (int i=1; i<=n; i++) {
            for (int j=1; j<=n; j++) {
                if (position.map(j)==i) {
                    output.print("Q");
                }
                else {
                    output.print(".");
                }
            }
            output.println();
        }
        output.println();
    }
            
}