/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

// This program writes all the solutions for the following problem:
// A riddle: In how many ways can you place 8 queens on a chess-
// board such that no one will threaten the other.
class RecursiveQueensRiddle {

    // The size of the chessboard
    private static final int SIZE = 8;
    
    /**
     * Solves the riddle.
     */
    public static void main(String[] args) {

        OutputWindow output = new OutputWindow("Queens Riddle");

        int[] position = new int[SIZE];
        long start = System.currentTimeMillis();
        complete(position, 0, output);
        long interval = System.currentTimeMillis()-start;
        System.out.println(interval);
    }
    
    // assumes that the first n-1 columns of position
    // are ok and try to complete the position
    static void complete(int[] position, int n, OutputWindow output) {
        if (n==SIZE) { // all colums are fine
            print(position, output);
            return;
        }
        for (position[n]=0; position[n]<SIZE; position[n]++) {
            if (isValid(position,n)) {
                complete(position, n+1, output);
            }
        }
    }

    // Checks if the position is valid till the column n
    // starting from column 0.
    private static boolean isValid(int[] position, int n) {
        for (int i=0; i<=n; i++) {
            for (int j=0; j<=n; j++) {
                if (i==j) {
                    continue;
                }
                if (position[i]==position[j] ||
                    Math.abs(i-j)==Math.abs(position[i]-position[j])) {
                    return false;
                }
            }
        }
        return true;
    }

    // prints the board position represented by a given position
    private static void print(int[] position, OutputWindow output) {
        for (int i=0; i<SIZE; i++) {
            for (int j=0; j<SIZE; j++) {
                if (position[j]==i) {
                    output.print("Q");
                }
                else {
                    output.print(".");
                }
            }
            output.println();
        }
        output.println();
    }

}