/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * A library of statistical methods.
 *
 * @author <A Href=londons@cs.huji.ac.il>Shmulik London</A>
 */
public class Statistics {

    /**
     * Returns the minimal value of the given population.
     * If the population is empty return positive infinity
     * (this is a useful convention sometimes used in mathematics)
     * @param population A population of values.
     * @return the minimal value of the given population.
     */
    public static double min(double[] population) {
        double min = Double.POSITIVE_INFINITY;
        for (int i=0; i<population.length; i++) {
            min = (population[i]<min) ? population[i] : min;
        }
        return min;
    }

    /**
     * Returns the maximal value of the given population.
     * If the population is empty return negative infinity
     * (this is a useful convention sometimes used in mathematics)
     * @param population A population of values.
     * @return the maximal value of the given population. 
     */
    public static double max(double[] population) {
        double max = Double.NEGATIVE_INFINITY;
        for (int i=0; i<population.length; i++) {
            max = (population[i]>max) ? population[i] : max;
        }
        return max;
    }

    /**
     * Returns the sum of values of a given population.
     * If the population is empty returns 0 (the sum of
     * an empty population of numbers is zero).
     * @param population A population of values.
     * @return the sum of values of a given population.
     */
    public static double sum(double[] population) {
        double sum = 0.0;
        for (int i=0; i<population.length; i++) {
            sum += population[i];
        }
        return sum;
    }

    /**
     * Returns the average value of a population.
     * If the population is empty returns Doulbe.NaN
     * (The average of an empty list of numbers is not
     * defined).
     * @param population A population of values.
     */
    public static double average(double[] population) {
        return sum(population)/population.length;
    }

}