/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * A vocabulary is a set of words. One can query the vocabulary
 * if a given word exists in this vocabulary.
 * 
 * @author <A Href=mailto:londons@idc.ac.il>Shmulik London</A>
 */
public class Vocabulary {
    
    // The list of words in accending order
    private String[] words;
    
    /**
     * Constructs a Vocabulary from a text file.
     * The text file should contains list of words in 
     * accending order, each word on a separate line.
     * @param filename The name of the file.
     */
    public Vocabulary(String filename) {
        int numberOfWords = countLines(filename);
        words = new String[numberOfWords];
        TextDocument document = new TextDocument(filename);
        for (int i=0; document.hasMoreLines(); i++) {
            words[i] = document.nextLine().trim();
        }
    }
    
    /**
     * Checks if the vocabulary contains a given word.
     * @param word A given word
     * @return true if the given word is founc in the
     * vocabulary.
     */
    public boolean contains(String word) {
        int lowerLimit = 0;
        int upperLimit = words.length-1;
        
        while (lowerLimit<=upperLimit) {
            int i = (upperLimit+lowerLimit)/2;
            if (word.equals(words[i])) {
                return true;
            }
            if (word.compareTo(words[i])<0) {
                upperLimit = i-1;
            }
            else {
                lowerLimit = i+1;
            }
        }
        return false;
    }
    
    // Counts the number of lines in a text line
    // filename - The name of the given text file
    private int countLines(String filename) {
        TextDocument document = new TextDocument(filename);
        int numberOfLines = 0;
        while (document.hasMoreLines()) {
            numberOfLines++;
            document.nextLine();
        }
        return numberOfLines;
    }
            
}
            
