/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * A library of arithmetical functions.
 *
 * @author Shmulik London
 */
public class Arithmetics {

    /**
     * Computes the factorial of a given number.
     * @param n A given integer.
     * @return The factorial of n.
     */
    public static long factorial(int n) {
        return (n<=1) ? 1 : n*factorial(n-1);
    }

    /**
     * Computes the greatest common divisor of two integers.
     * @param a,b Integers.
     * @return The GCD of a and b.
     */
    public static int gcd(int a, int b) {
        if (a==b) {
            return a;
        }
        a = Math.abs(a);
        b = Math.abs(b);
        return gcd(Math.abs(a-b), Math.min(a,b));
    }

    /**
     * Computes the n'th fibonacci number.
     * @param n The index of the required fibonacci number.
     * @return The n'th fibonacci number.
     * @exception IllegalArgumentException if n is negative
     */
    public static int fibonacci(int n) {
        if (n<0) {
            throw new IllegalArgumentException(
                "Index must be non-negative");
        }
        return (n<=1) ? 1 : fibonacci(n-1)+fibonacci(n-2); 
    }

}
