/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * This program simulates graphically the problem of the 'Towers of Hanoi'.
 *
 * @author Shmulik London londons@cs.huji.ac.il
 */
class TowersOfHanoi {

    /**
     * Simulates the 'Towers of Hanoi' problem with a
     * given number of disks.<BR>
     * Usage:<TT> java TowersOfHanoi &lt;number of disks&gt; </TT>
     */
    public static void main(String[] args) {
        try {
            int numberOfDisks = Integer.parseInt(args[0]);
            Pile[] piles = new Pile[3];
            for (int i=0; i<piles.length; i++) {
                piles[i] = new Pile();
            }
            for (int i=numberOfDisks+1; i>1; i--) {
                piles[0].addDisk(new Disk(i));
            }
            solve(piles, piles[0].getNumberOfDisks(), 0, 2, 1);
        } catch (Exception e) {
            System.err.println("Illegal usage!");
            System.err.println("Usage: java TowersOfHanoi <number of disks>");
            System.err.println("Example: java TowersOfHanoi 5");
        }
    }

    //
    // Solves the problem of transferring a given number of disks from
    // a pile to another pile using a third pile.
    // piles - The piles
    // n - The number of disks to be transfered
    // from - The index of the pile from which the disks should be removed
    // to - The index of the pile to which the disks should be transfered to.
    // using - The index of the auxilary pile.
    private static void solve(Pile[] piles, int n, int from, int to, int using) {
        if (n==1) {
            piles[to].addDisk(piles[from].removeDisk());
            return;
        }
        solve(piles, n-1, from, using, to);
        solve(piles, 1, from, to, using);
        solve(piles, n-1, using, to, from);
    }

}